@@ -1,15 +1,18 @@
 import json
 import logging
-from typing import Optional
+from typing import Optional, List
 
 import jsonpickle
 import requests
 
 import src
+from src.api.entities.cliente import Cliente
 from src.entities.invoice import Invoice
+from src.entities.log import Log
 from src.entities.order import Queue, Order
+from src.entities.orderb2b import OrderB2B
 from src.entities.price import Price
-from src.entities.response import CatalogoResponse, PriceResponse, InvoiceResponse, TrackingResponse
+from src.entities.response import CatalogoResponse, PriceResponse, InvoiceResponse, TrackingResponse, StockResponse, ClientResponse
 from src.entities.tracking import Tracking
 
 logger = logging.getLogger()
@@ -62,7 +65,9 @@ def send_stocks(url, body, token):
                    'Accept': 'text/html',
                    'access-token': token}
 
-        response = requests.put(url, json=body, headers=headers)
+        logger.info(body)
+        json_stock = jsonpickle.encode(body, unpicklable=False)
+        response = requests.put(url, json=json.loads(json_stock), headers=headers)
 
         if response.ok:
             return response.json(), response.status_code
@@ -71,10 +76,28 @@ def send_stocks(url, body, token):
                 return response.json(), response.status_code
 
     except Exception as ex:
-        logger.error(str(ex), exc_info=True)
+        logger.error(str(ex))
         return None, response.status_code
 
 
+def send_stocks_ud(url, body, token) -> List[StockResponse]:
+    logger.debug(f"POST: {url}")
+    try:
+        headers = {
+            'Content-type': 'application/json',
+            'Accept': 'application/json',
+            'access-token': token
+        }
+
+        logger.info(body)
+        json_stock = jsonpickle.encode(body, unpicklable=False)
+        response = requests.post(url, json=json.loads(json_stock), headers=headers)
+        return [StockResponse(**c) for c in jsonpickle.decode(response.content)]
+
+    except Exception as e:
+        return [StockResponse([body[0].codigo_erp], "-1", str(e), None)]
+
+
 def post_prices(url, price: Price, token) -> PriceResponse:
     try:
         headers = {'Content-type': 'application/json',
@@ -93,7 +116,7 @@ def post_prices(url, price: Price, token) -> PriceResponse:
         return PriceResponse([price.codigo_erp], 3, str(ex), '', '')
 
 
-def get_order_queue(url: str, token: str, status: str, limit: int) -> list[Queue]:
+def get_order_queue(url: str, token: str, status: str, limit: int) -> List[Queue]:
     queue = []
     try:
         response = requests.get(url.format(status), headers={'Accept': 'application/json', 'access-token': token}, params={'limit': limit})
@@ -125,6 +148,22 @@ def get_order(url: str, token: str, order_id: int) -> Order:
     return order
 
 
+def get_order_b2b(url: str, token: str, order_id: int) -> OrderB2B:
+    order = None
+    try:
+        response = requests.get(url.format(order_id), headers={'Accept': 'application/json', 'access-token': token})
+        if response.ok:
+            obj = jsonpickle.decode(response.content)
+            order = OrderB2B(**obj)
+        else:
+            logger.warning(f'Retorno sem sucesso {response.status_code} - {response.url}')
+    except Exception as ex:
+        logger.error(f'Erro ao realizar GET na api okvendas {url} - {str(ex)}')
+        raise
+
+    return order
+
+
 def put_order_erp_code(url: str, token: str, order_id: int, order_erp_id: str) -> bool:
     try:
         response = requests.put(url, headers={'Accept': 'application/json', 'access-token': token}, params={'id': order_id, 'codigo_erp': order_erp_id})
@@ -152,10 +191,11 @@ def put_client_erp_code(url: str, token: str, body: dict) -> bool:
         return False
 
 
-def put_protocol_orders(order_id) -> bool:
-    url = src.client_data.get('url_api') + f'/pedido/integrado/{order_id}'
+def put_protocol_orders(protocolos: List[str]) -> bool:
+    url = src.client_data.get('url_api') + f'/pedido/fila'
     try:
-        response = requests.put(url, headers={'Accept': 'application/json', 'access-token': src.client_data.get('token_api')})
+        json_protocolos = jsonpickle.encode(protocolos)
+        response = requests.put(url, json=json.loads(json_protocolos), headers={'Accept': 'application/json', 'access-token': src.client_data.get('token_api')})
         if response.ok:
             return True
         else:
@@ -189,7 +229,10 @@ def post_invoices(url: str, invoice: Invoice, token) -> Optional[InvoiceResponse
             return None
         else:
             err = jsonpickle.decode(str(response.text))
-            return InvoiceResponse(**err)
+            invoice_response = InvoiceResponse(**err)
+            if '_okvendas' in invoice_response.message or '_openkuget' in invoice_response.message:
+                invoice_response.message = 'Erro interno no servidor. Entre em contato com o suporte'
+            return invoice_response
 
     except Exception as ex:
         return InvoiceResponse(3, str(ex))
@@ -218,7 +261,52 @@ def post_tracking(url: str, tracking: Tracking, token) -> Optional[TrackingRespo
             return None
         else:
             err = jsonpickle.decode(str(response.text))
-            return TrackingResponse(**err)
+            tracking_response = TrackingResponse(**err)
+            if '_okvendas' in tracking_response.message or '_openkuget' in tracking_response.message:
+                tracking_response.message = 'Erro interno no servidor. Entre em contato com o suporte'
+            return tracking_response
 
     except Exception as ex:
         return TrackingResponse(3, str(ex))
+
+
+def post_log(log: Log) -> bool:
+    try:
+        if not src.is_dev:
+            url = f'{src.client_data.get("url_api")}/log/logIntegracao'
+            headers = {
+                'Content-type': 'application/json',
+                'Accept': 'application/json',
+                'access-token': src.client_data.get('token_api')
+            }
+
+            json_log = jsonpickle.encode([log], unpicklable=False)
+            print(json_log)
+            response = requests.post(url, json=json.loads(json_log), headers=headers)
+            if response.ok:
+                return True
+
+            return False
+        else:
+            return True
+    except Exception:
+        return False
+
+
+def post_clients(clients: List[Cliente]) -> List[ClientResponse]:
+    url = f'{src.client_data.get("url_api")}/cliente/clientes'
+    headers = {
+        'Content-type': 'application/json',
+        'Accept': 'application/json',
+        'access-token': src.client_data.get('token_api')
+    }
+
+    try:
+        payload = jsonpickle.encode(clients, unpicklable=False)
+        response = requests.put(url, json=json.loads(payload), headers=headers)
+        results = jsonpickle.decode(response.text)
+        return [ClientResponse(**r) for r in results]
+
+    except Exception as e:
+        return [ClientResponse([c.cpf or c.cnpj for c in clients], 3, str(e))]
+
