@@ -8,6 +8,7 @@ import src.database.connection as database
 import src.database.utils as utils
 from src.database import queries
 from src.database.utils import DatabaseConfig
+from src.entities.orderb2b import OrderB2B
 from src.entities.tracking import Tracking
 from src.entities.invoice import Invoice
 from src.entities.order import Order, Queue
@@ -30,27 +31,30 @@ queue_status = {
 def define_job_start(job_config: dict) -> None:
     global current_job
     current_job = job_config.get('job_name')
-    if current_job == 'internaliza_pedidos_job':  # Inicia o job a partir dos pedidos AgPagamento
+    if current_job == 'internaliza_pedidos_job' or current_job == 'internaliza_pedidos_b2b_job':  # Inicia o job a partir dos pedidos AgPagamento
         job_orders(job_config, True)
-    else:  # Inicia o job a partir dos pedidos Confirmados
+    elif current_job == 'internaliza_pedidos_pagos_job' or current_job == 'internaliza_pedidos_pagos_b2b_job':  # Inicia o job a partir do pedidos pagos
         job_orders(job_config)
 
 
 def job_orders(job_config: dict, start_at_pending: bool = False) -> None:
     try:
         db_config = utils.get_database_config(job_config)
-        if start_at_pending:
-            process_order_queue(queue_status.get('pending'), db_config, True)
+        if not current_job.__contains__('b2b'):
+            if start_at_pending:
+                process_order_queue(queue_status.get('pending'), db_config, True)
 
-        process_order_queue(queue_status.get('paid'), db_config, True)
+            process_order_queue(queue_status.get('paid'), db_config, True)
 
-        process_order_queue(queue_status.get('canceled'), db_config)
+            process_order_queue(queue_status.get('canceled'), db_config)
 
-        # process_order_queue(queue_status.get('invoiced'), db_config)
+        else:
+            if start_at_pending:
+                process_b2b_order_queue(queue_status.get('pending'), db_config, True)
 
-        process_order_queue(queue_status.get('shipped'), db_config)
+            process_b2b_order_queue(queue_status.get('paid'), db_config, True)
 
-        process_order_queue(queue_status.get('delivered'), db_config)
+            process_b2b_order_queue(queue_status.get('canceled'), db_config)
 
     except Exception as e:
         logger.error(current_job + f' | Erro ao inicializar job: {str(e)}')
@@ -100,6 +104,8 @@ def job_send_erp_tracking(job_config: dict):
 
 def process_order_queue(status: str, db_config: DatabaseConfig, status_to_insert: bool = False) -> None:
     try:
+        print()
+        print()
         logger.info(current_job + f' | Consultando fila de pedidos no status {status}')
         queue = api_okvendas.get_order_queue(
             url=src.client_data.get('url_api') + '/pedido/fila/{0}',
@@ -117,7 +123,8 @@ def process_order_queue(status: str, db_config: DatabaseConfig, status_to_insert
                 order = api_okvendas.get_order(
                     url=src.client_data.get('url_api') + '/pedido/{0}',
                     token=src.client_data.get('token_api'),
-                    order_id=q_order.order_id)
+                    order_id=q_order.order_id
+                )
 
                 if order.erp_code is not None and order.erp_code != '':  # Pedido integrado anteriormente
 
@@ -162,10 +169,74 @@ def process_order_queue(status: str, db_config: DatabaseConfig, status_to_insert
         api_okvendas.post_log(Log(current_job + f' | Erro ao inicializar job de processamento de pedidos do status {status}: {str(e)}', datetime.now().isoformat(), status, 'PEDIDO'))
 
 
-def protocol_order(db_config: DatabaseConfig, order: Order, queue_order: Queue, order_erp_id: str = '', client_erp_id: str = '') -> None:
+def process_b2b_order_queue(status: str, db_config: DatabaseConfig, status_to_insert: bool = False) -> None:
+    try:
+        logger.info(current_job + f' | Consultando fila de pedidos no status {status}')
+        queue = api_okvendas.get_order_queue(
+            url=src.client_data.get('url_api') + '/pedido/fila/{0}',
+            token=src.client_data.get('token_api'),
+            status=status,
+            limit=default_limit)
+
+        qty = 0
+        for q_order in queue:
+            try:
+                sleep(0.5)
+                qty = qty + 1
+                print()
+                logger.info(current_job + f' | Iniciando processamento ({qty} de {len(queue)}) pedido {q_order.order_id}')
+                order = api_okvendas.get_order_b2b(
+                    url=src.client_data.get('url_api') + '/pedidoB2B/{0}',
+                    token=src.client_data.get('token_api'),
+                    order_id=q_order.order_id
+                )
+
+                if order.erp_code is not None and order.erp_code != '':  # Pedido integrado anteriormente
+
+                    if check_order_existence(db_config, order.order_id):
+                        logger.info(current_job + ' | Pedido ja integrado com o ERP, chamando procedure de atualizacao...')
+                        if call_update_order_procedure(db_config, order):
+                            logger.info(current_job + ' | Pedido atualizado com sucesso')
+                            protocol_order(db_config=db_config, order=order, queue_order=q_order, order_erp_id='', client_erp_id='')
+                    else:
+                        logger.warning(current_job + f' | Pedido {order.order_id} nao existe no banco semaforo porem ja foi integrado previamente. Protocolando pedido...')
+                        protocol_non_existent_order(q_order)
+
+                else:  # Pedido nao integrado anteriormente
+
+                    if check_order_existence(db_config, order.order_id):  # Pedido existente no banco semaforo
+
+                        logger.info(current_job + ' | Pedido já existente no banco semáforo, porem nao integrado com erp. Chamando procedures')
+                        sp_success, client_erp_id, order_erp_id = call_order_procedures(db_config, q_order.order_id)
+                        if sp_success:
+                            logger.info(current_job + ' | Chamadas das procedures executadas com sucesso, protocolando pedido...')
+                            protocol_order(db_config=db_config, order=order, queue_order=q_order, order_erp_id=order_erp_id, client_erp_id=client_erp_id)
+
+                    elif status_to_insert:  # Pedido nao existe no semaforo e esta em status de internalizacao (pending e paid)
+
+                        logger.info(current_job + ' | Inserindo novo pedido no banco semaforo')
+                        inserted = insert_temp_b2b_order(order, db_config)
+                        if inserted:
+                            logger.info(current_job + ' | Pedido inserido com sucesso, chamando procedures...')
+                            sp_success, client_erp_id, order_erp_id = call_order_procedures(db_config, q_order.order_id)
+                            if sp_success:
+                                logger.info(current_job + ' | Chamadas das procedures executadas com sucesso, protocolando pedido...')
+                                protocol_order(db_config=db_config, order=order, queue_order=q_order, order_erp_id=order_erp_id, client_erp_id=client_erp_id)
+
+                    else:  # Pedido nao existe no semaforo e nao esta em status de internalizacao (pending e paid)
+                        logger.warning(current_job + ' | Pedido nao existente no banco semaforo e nao se encontra em status de internalizacao')
+
+            except Exception as e:
+                logger.error(current_job + f' | Erro no processamento do pedido {q_order.order_id}: {str(e)}')
+                api_okvendas.post_log(Log(current_job + f' | Erro no processamento do pedido {q_order.order_id}: {str(e)}', datetime.now().isoformat(), str(q_order.order_id), 'PEDIDO'))
+    except Exception as e:
+        logger.error(current_job + f' | Erro ao inicializar job de processamento de pedidos: {str(e)}')
+        api_okvendas.post_log(Log(current_job + f' | Erro ao inicializar job de processamento de pedidos do status {status}: {str(e)}', datetime.now().isoformat(), status, 'PEDIDO'))
+
+
+def protocol_order(db_config: DatabaseConfig, order, queue_order: Queue, order_erp_id: str = '', client_erp_id: str = '') -> None:
     db = database.Connection(db_config)
     try:
-        print()
         if order_erp_id != '':
             logger.info(current_job + f' | Protocolando pedido com codigo_referencia {order_erp_id}')
             updated_order_code = api_okvendas.put_order_erp_code(src.client_data.get('url_api') + '/pedido/integradoERP',
@@ -177,9 +248,16 @@ def protocol_order(db_config: DatabaseConfig, order: Order, queue_order: Queue,
             else:
                 logger.warning(current_job + ' | Falha ao atualizar o Codigo Erp do pedido via api OkVendas')
 
-        print()
         if client_erp_id != '':
-            logger.info(current_job + f' | Protocolando cliente com codigo_referencia {client_erp_id}')
+            logger.info(current_job + f' | Salvando codigo do erp do cliente no banco semaforo {client_erp_id}')
+            conn = db.get_conect()
+            cursor = conn.cursor()
+            cursor.execute(queries.get_client_protocol_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [client_erp_id, order.order_id]))
+            cursor.close()
+            conn.commit()
+            conn.close()
+
+            logger.info(current_job + f' | Protocolando cliente via api okvendas com codigo_referencia {client_erp_id}')
             updated_client_code = api_okvendas.put_client_erp_code(src.client_data.get('url_api') + '/cliente/codigo',
                                                                    src.client_data.get('token_api'),
                                                                    {
@@ -191,7 +269,6 @@ def protocol_order(db_config: DatabaseConfig, order: Order, queue_order: Queue,
             else:
                 logger.warning(current_job + ' | Falha ao atualizar o Codigo Erp do cliente via api OkVendas')
 
-        print()
         logger.info(current_job + f' | Removendo pedido da fila pelo protocolo {queue_order.protocol}')
         protocoled_order = api_okvendas.put_protocol_orders([queue_order.protocol])
         if protocoled_order:
@@ -231,63 +308,68 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
         step = 'conexao'
         cursor = conn.cursor()
 
-        if order.user.erp_code is not None and order.user.erp_code != '':
+        existent_client = None
+        if order.user.erp_code is not None and order.user.erp_code != '' and order.user.erp_code != '0':  # Por padrao o erp_code vem = '0' da api okvendas
             cursor.execute(queries.get_query_client_erp(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.erp_code]))
             existent_client = cursor.fetchone()
-            if existent_client is None:
-                # insere cliente
-                step = 'insere cliente'
-                logger.info(f'\tPedido {order.order_id}: Inserindo cliente')
-                cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
-                    order.user.name or order.user.company_name,
-                    order.user.company_name or order.user.name,
-                    order.user.cpf,
-                    order.user.cnpj,
-                    order.user.email,
-                    order.user.residential_phone,
-                    order.user.mobile_phone,
-                    order.user.address.zipcode,
-                    order.user.address.address_type,
-                    order.user.address.address_line,
-                    order.user.address.number,
-                    order.user.address.complement,
-                    order.user.address.neighbourhood or " ",
-                    order.user.address.city,
-                    order.user.address.state,
-                    order.user.address.reference]))
-            else:
-                # update no cliente existente
-                step = 'update cliente'
-                logger.info(f'\tPedido {order.order_id}: Atualizando cliente existente')
-                cursor.execute(queries.get_update_client_sql(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
-                    order.user.name,
-                    order.user.company_name or order.user.name,
-                    order.user.cpf,
-                    order.user.cnpj,
-                    order.user.email,
-                    order.user.residential_phone,
-                    order.user.mobile_phone,
-                    order.user.address.zipcode,
-                    order.user.address.address_type,
-                    order.user.address.address_line,
-                    order.user.address.number,
-                    order.user.address.complement,
-                    order.user.address.neighbourhood,
-                    order.user.address.city,
-                    order.user.address.state,
-                    order.user.address.reference,
-                    order.user.erp_code]))
 
-            if cursor.rowcount > 0:
-                logger.info(current_job + f' | Cliente inserido/atualizado para o pedido {order.order_id}')
-                cursor.execute(queries.get_query_client(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.email]))
-                (client_id, ) = cursor.fetchone()
-                if client_id is None or client_id <= 0:
-                    cursor.close()
-                    raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
-            else:
+        if existent_client is None:
+            # insere cliente
+            step = 'insere cliente'
+            logger.info(f'\tPedido {order.order_id}: Inserindo cliente')
+            cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
+                order.user.name or order.user.company_name,
+                order.user.company_name or order.user.name,
+                order.user.cpf,
+                order.user.cnpj,
+                order.user.email,
+                order.user.residential_phone,
+                order.user.mobile_phone,
+                order.user.address.zipcode,
+                order.user.address.address_type,
+                order.user.address.address_line,
+                order.user.address.number,
+                order.user.address.complement,
+                order.user.address.neighbourhood or " ",
+                order.user.address.city,
+                order.user.address.state,
+                order.user.address.reference,
+                'IN'
+            ]))
+        else:
+            # update no cliente existente
+            step = 'update cliente'
+            logger.info(f'\tPedido {order.order_id}: Atualizando cliente existente')
+            cursor.execute(queries.get_update_client_sql(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
+                order.user.name,
+                order.user.company_name or order.user.name,
+                order.user.cpf,
+                order.user.cnpj,
+                order.user.email,
+                order.user.residential_phone,
+                order.user.mobile_phone,
+                order.user.address.zipcode,
+                order.user.address.address_type,
+                order.user.address.address_line,
+                order.user.address.number,
+                order.user.address.complement,
+                order.user.address.neighbourhood,
+                order.user.address.city,
+                order.user.address.state,
+                order.user.address.reference,
+                order.user.erp_code
+            ]))
+
+        if cursor.rowcount > 0:
+            logger.info(current_job + f' | Cliente inserido/atualizado para o pedido {order.order_id}')
+            cursor.execute(queries.get_query_client_id(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.email]))
+            (client_id, ) = cursor.fetchone()
+            if client_id is None or client_id <= 0:
                 cursor.close()
-                raise Exception('O cliente nao foi inserido')
+                raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
+        else:
+            cursor.close()
+            raise Exception('O cliente nao foi inserido')
 
         # insere pedido
         step = 'insere pedido'
@@ -311,7 +393,8 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
             str(datetime.strptime(order.delivery_forecast.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.delivery_forecast is not None else ''),
             order.carrier,
             order.shipping_mode,
-            order.channel_id]))
+            order.channel_id
+        ]))
 
         if cursor.rowcount > 0:
             logger.info(current_job + f' | Pedido {order.order_id} inserido')
@@ -336,7 +419,147 @@ def insert_temp_order(order: Order, db_config: DatabaseConfig) -> bool:
                 item.ean,
                 item.value,
                 item.discount,
-                item.freight_value]))
+                item.freight_value
+            ]))
+
+        cursor.close()
+        conn.commit()
+        conn.close()
+        return True
+    except Exception as e:
+        logger.error(current_job + f' | Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}')
+        api_okvendas.post_log(Log(current_job + f' | Passo {step} - Erro durante a inserção dos dados do pedido {order.order_id}: {str(e)}', datetime.now().isoformat(), str(order.order_id), 'PEDIDO'))
+        conn.rollback()
+        conn.close()
+        return False
+
+
+def insert_temp_b2b_order(order: OrderB2B, db_config: DatabaseConfig) -> bool:
+    step = ''
+    db = database.Connection(db_config)
+    conn = db.get_conect()
+    try:
+        step = 'conexao'
+        cursor = conn.cursor()
+
+        existent_client = None
+        if order.user.erp_code is not None and order.user.erp_code != '' and order.user.erp_code != '0':  # Por padrao o erp_code vem = '0' da api okvendas
+            cursor.execute(queries.get_query_client_erp(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.erp_code]))
+            existent_client = cursor.fetchone()
+
+        if existent_client is None:
+            # insere cliente
+            step = 'insere cliente'
+            logger.info(f'\tPedido {order.order_id}: Inserindo cliente')
+            cursor.execute(queries.get_insert_client_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
+                order.user.name or order.user.company_name,
+                order.user.company_name or order.user.name,
+                order.user.cpf,
+                order.user.cnpj,
+                order.user.email,
+                order.user.residential_phone,
+                order.user.mobile_phone,
+                order.user.address.zipcode,
+                order.user.address.address_type,
+                order.user.address.address_line,
+                order.user.address.number,
+                order.user.address.complement,
+                order.user.address.neighbourhood or " ",
+                order.user.address.city,
+                order.user.address.state,
+                order.user.address.reference,
+                'IN'
+            ]))
+        else:
+            # update no cliente existente
+            step = 'update cliente'
+            logger.info(f'\tPedido {order.order_id}: Atualizando cliente existente')
+            cursor.execute(queries.get_update_client_sql(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
+                order.user.name,
+                order.user.company_name or order.user.name,
+                order.user.cpf,
+                order.user.cnpj,
+                order.user.email,
+                order.user.residential_phone,
+                order.user.mobile_phone,
+                order.user.address.zipcode,
+                order.user.address.address_type,
+                order.user.address.address_line,
+                order.user.address.number,
+                order.user.address.complement,
+                order.user.address.neighbourhood,
+                order.user.address.city,
+                order.user.address.state,
+                order.user.address.reference,
+                order.user.erp_code
+            ]))
+
+        if cursor.rowcount > 0:
+            logger.info(current_job + f' | Cliente inserido/atualizado para o pedido {order.order_id}')
+            cursor.execute(queries.get_query_client_id(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.user.email]))
+            (client_id, ) = cursor.fetchone()
+            if client_id is None or client_id <= 0:
+                cursor.close()
+                raise Exception('Nao foi possivel obter o cliente inserido do banco de dados')
+        else:
+            cursor.close()
+            raise Exception('O cliente nao foi inserido')
+
+        # insere pedido
+        step = 'insere pedido'
+        logger.info(f'\tPedido {order.order_id}: Inserindo cabecalho pedido')
+        cursor.execute(queries.get_insert_b2b_order_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
+            order.order_id,
+            order.sale_order_id,
+            str(datetime.strptime(order.order_date.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.order_date is not None else ''),
+            order.status,
+            client_id,
+            order.total_amount,
+            order.discount_amount,
+            order.freight_amount,
+            0.0,
+            str(datetime.strptime(order.payments[0].bonds[0].paid_date.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if len(order.payments) > 0 and len(order.payments[0].bonds) > 0 and order.paid_date is not None else ''),
+            order.payment_type,
+            order.flag,
+            order.installments,
+            order.erp_payment_condition,
+            order.tracking_code,
+            str(datetime.strptime(order.delivery_forecast.replace('T', ' '), '%Y-%m-%d %H:%M:%S') if order.delivery_forecast is not None else ''),
+            order.carrier,
+            '',
+            order.channel_id,
+            order.erp_representative
+        ]))
+
+        if cursor.rowcount > 0:
+            logger.info(current_job + f' | Pedido {order.order_id} inserido')
+            cursor.execute(queries.get_query_order(db_config.db_type), queries.get_command_parameter(db_config.db_type, [order.order_id]))
+            (order_id, ) = cursor.fetchone()
+            if order_id is None or order_id <= 0:
+                cursor.close()
+                raise Exception('Nao foi possivel obter o pedido inserido no banco de dados')
+        else:
+            cursor.close()
+            raise Exception('O cliente nao foi inserido')
+
+        # insere itens
+        step = 'insere itens'
+        logger.info(f'\tPedido {order.order_id} com id semaforo {order_id}: Inserindo itens do pedido')
+        for item in order.items:
+            cursor.execute(queries.get_insert_b2b_order_items_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
+                order_id,
+                item.sku,
+                item.erp_code,
+                item.quantity,
+                item.ean,
+                item.value,
+                item.discount,
+                0,  # Valor do frete não necessario, usar o do cabeçalho
+                item.selling_branch_cnpj,
+                item.branch_erp_code,
+                item.expedition_branch,
+                item.invoice_branch
+            ]))
 
         cursor.close()
         conn.commit()
@@ -401,7 +624,7 @@ def call_order_procedures(db_config: DatabaseConfig, order_id: int) -> (bool, st
         conn.close()
 
 
-def call_update_order_procedure(db_config: DatabaseConfig, order: Order) -> bool:
+def call_update_order_procedure(db_config: DatabaseConfig, order) -> bool:
     success = True
     db = database.Connection(db_config)
     conn = db.get_conect()
