@@ -1,3 +1,4 @@
+from datetime import datetime
 import logging
 import src.database.connection as database
 import src.database.utils as utils
@@ -6,6 +7,8 @@ import src.api.okvendas as api_okvendas
 from src.api import slack
 from src.database import queries
 import src
+import time
+from src.entities.log import Log
 
 logger = logging.getLogger()
 
@@ -30,7 +33,8 @@ def job_insert_stock_semaphore(job_config_dict: dict):
             cursor.execute(db_config.sql)
             logger.info(job_config_dict.get('job_name') + f' | {cursor.rowcount} estoques inseridos no banco semaforo')
         except Exception as ex:
-            logger.error(job_config_dict.get('job_name') + ' | Erro ' + str(ex), exc_info=True)
+            logger.error(job_config_dict.get('job_name') + ' | Erro ' + str(ex))
+            api_okvendas.post_log(Log(job_config_dict.get('job_name') + ' | Erro ao inserir estoques no banco semaforo' + str(ex), datetime.now().isoformat(), '', 'ESTOQUE'))
 
         cursor.close()
         conn.commit()
@@ -45,38 +49,39 @@ def job_send_stocks(job_config_dict: dict):
     """
     db_config = utils.get_database_config(job_config_dict)
     stocks = query_stocks(db_config)
-    p_size = len(stocks) if stocks is not None else 0
     atualizados = []
-    if p_size > 0:
-        logger.debug("Total de produtos atualiza estoque API: {}".format(p_size))
+    if len(stocks) or 0 > 0:
+        logger.info(job_config_dict.get('job_name') + f"Total de estoques a serem atualizados: {len(stocks)}")
         for stock in stocks:
-            response, status_code = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoque', stock, src.client_data.get('token_api'))
+            try:
+                time.sleep(1)
+                response, status_code = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoque', stock, src.client_data.get('token_api'))
 
-            if response is not None:
-                logger.debug('Atualizando produtos no semaforo')
-                conexao = database.Connection(db_config)
-                conn = conexao.get_conect()
-                cursor = conn.cursor()
+                if response is not None:
+                    conexao = database.Connection(db_config)
+                    conn = conexao.get_conect()
+                    cursor = conn.cursor()
 
-                codigo_erp = response['codigo_erp']
-                if response['Status'] == 1 or response['Status'] == 'Success':
-                    try:
+                    codigo_erp = response['codigo_erp']
+                    if response['Status'] == 1 or response['Status'] == 'Success':
                         sql_protocolar_estoque = queries.get_stock_protocol_command(db_config.db_type)
                         cursor.execute(sql_protocolar_estoque, queries.get_command_parameter(db_config.db_type, [codigo_erp]))
                         atualizados.append(response['codigo_erp'])
-                    except Exception as ex:
-                        logger.error(str(ex), exc_info=True)
-                else:
-                    logger.warning(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}')
+                    else:
+                        logger.warning(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}')
+                        api_okvendas.post_log(Log(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque para o sku {codigo_erp}', datetime.now().isoformat(), codigo_erp, 'ESTOQUE'))
 
-                cursor.close()
-                conn.commit()
-                conn.close()
+                    cursor.close()
+                    conn.commit()
+                    conn.close()
+            except Exception as e:
+                logger.error(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque para o sku {stock.get("codigo_erp")}: {str(e)}')
+                api_okvendas.post_log(Log(
+                    job_config_dict.get('job_name') + f' | Erro ao atualizar estoque do sku {stock.get("codigo_erp")}: {str(e)}', datetime.now().isoformat(), stock.get('codigo_erp'), 'ESTOQUE'))
 
-        total = len(atualizados)
-        logger.debug('Atualizado estoques no semaforo: {}'.format(total))
+        logger.info(job_config_dict.get('job_name') + f'Atualizado estoques no semaforo: {len(atualizados) or 0}')
     else:
-        logger.warning("Nao ha produtos para atualizar estoque")
+        logger.warning(job_config_dict.get('job_name') + "Nao ha produtos para atualizar estoque no momento")
 
 
 def job_send_stocks_ud(job_config_dict: dict):
@@ -90,38 +95,34 @@ def job_send_stocks_ud(job_config_dict: dict):
     p_size = len(stocks) if stocks is not None else 0
 
     if p_size > 0:
-        logger.debug("Total de produtos atualiza estoque API: {}".format(p_size))
-        jsn_prods = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoqueUnidadeDistribuicao', stocks, src.client_data.get('token_api'))
-
-        atualizados = []
-
-        if jsn_prods is not None:
-            logger.debug('Atualizando produtos no semaforo')
-            conexao = database.Connection(db_config)
-            conn = conexao.get_conect()
-            cursor = conn.cursor()
-
-            # Identifiers, Status, Message, Protocolo
-            for p in jsn_prods:
-                if p['Status'] == 1:
-                    cod_erp = p['Identifiers'][0]
+        time.sleep(0.5)
+        logger.debug(job_config_dict.get('job_name') + f' | Total de produtos para atualizar estoque: {p_size}')
+        count = 0
+        for stock in stocks:
+            codigo_erp = stock["codigo_erp"]
+            response = api_okvendas.send_stocks_ud(src.client_data.get('url_api') + '/catalogo/estoqueUnidadeDistribuicao', [stock], src.client_data.get('token_api'))
+            for res in response:
+                if res.status == 1:
                     try:
+                        logger.info(job_config_dict.get('job_name') + f' | Atualizando produto {codigo_erp} no semaforo')
+                        conn = database.Connection(db_config).get_conect()
+                        cursor = conn.cursor()
+
                         sql_protocolar_estoque = queries.get_stock_protocol_command(db_config.db_type)
-                        cursor.execute(sql_protocolar_estoque, queries.get_command_parameter(db_config.db_type, [cod_erp]))
-                        atualizados.append(cod_erp)
-                    except Exception as ex:
-                        logger.error(str(ex), exc_info=True)
+                        cursor.execute(sql_protocolar_estoque, queries.get_command_parameter(db_config.db_type, [codigo_erp]))
+
+                        cursor.close()
+                        conn.commit()
+                        conn.close()
+                        count = count + 1
+                    except Exception as e:
+                        logger.error(job_config_dict.get('job_name') + f' | Erro ao protocolar estoque do sku {codigo_erp}: {str(e)}')
                 else:
-                    logger.error(f'Erro ao atualizar estoque {p["Identifiers"][0]} erro da api: ' + p['Message'])
+                    logger.error(job_config_dict.get('job_name') + f' | Erro ao atualizar estoque {codigo_erp} erro da api: ' + res.message)
 
-            cursor.close()
-            conn.commit()
-            conn.close()
-
-            total = len(atualizados)
-            logger.debug('Atualizado produtos no semaforo: {}'.format(total))
+        logger.info(job_config_dict.get('job_name') + f' | Produtos protocolados no semaforo: {count}')
     else:
-        logger.warning("Nao ha produtos para atualizar estoque")
+        logger.warning(job_config_dict.get('job_name') + f' | Nao ha produtos para atualizar estoque')
 
 
 def query_stocks_ud(db_config: DatabaseConfig):
@@ -200,7 +201,7 @@ def stock_dict(produtos):
     lista = []
     for row in produtos:
         pdict = {
-            'codigo_erp': row['CODIGO_ERP'],
+            'codigo_erp': str(row['CODIGO_ERP']),
             'quantidade': int(row['QUANTIDADE'])
         }
         lista.append(pdict)
@@ -212,9 +213,9 @@ def stock_ud_dict(produtos):
     lista = []
     for row in produtos:
         pdict = {
-            'unidade_distribuicao': row[0],
-            'codigo_erp': row[1],
-            'quantidade_total': int(row[2]),
+            'unidade_distribuicao': row['CODIGO_UNIDADE_DISTRIBUICAO'],
+            'codigo_erp': row['CODIGO_ERP'],
+            'quantidade_total': int(row['QUANTIDADE']),
             'parceiro': 1
         }
         lista.append(pdict)
